# modal

## Summary

A series of modal dialog api like `toast`, `alert`, `confirm` and `prompt`.

## API

### toast(options)

#### Arguments

* `options`*(object)*: some options.
  * `message`*(string)*: the message that the toast shows.
  * `duration`*(number)*: the duration(seconds) that the toast shows.


##### Example

```javascript
var modal = require('@weex-module/modal');
modal.toast({'message': 'I am toast!', 'duration': 1});
```

### alert(options, callback)

#### Arguments

* `options`*(object)*: some options.
  * `message`*(string)*: the message that the alert shows.
  * `okTitle`*(string)*: the title of alert button.
* `callback`*(function)*: callback when complete.

#### Example

```javascript
var arg1 = 'I am alert!';
var arg2 = 'I am ok';
var modal = require('@weex-module/modal');
modal.alert({
  message: arg1,
  okTitle: arg2
}, function(e) {
  // TODO after the alert is complete.
})
```
    
### confirm(options, callback)

#### Arguments

* `options`*(object)*: some options.
  * `message`*(string)*: the message that the confirm shows.
  * `okTitle`*(string)*: the title of confirm button.
  * `cancelTitle`*(string)*: the title of cancel button.
* `callback`*(function)*: callback when complete.

This method has a callback function whose arguments will be:

* `e`*(object)*: a event object.
  * `status`*(string)*: the title of the button that clicked by user.
  
#### Example

```javascript
var arg1 = 'I am alert!'
var arg2 = 'I am ok'
var arg3 = 'I am cancel'
var modal = require('@weex-module/modal');
modal.confirm({
  message: arg1,
  okTitle: arg2,
  cancelTitle: arg3
}, function(e) {
  nativeLog(e.status)
  // TODO after the confirm is complete.
});
```

### prompt(options, callback)

#### Arguments

* `options`*(object)*: some options.
  * `message`*(string)*: the message that the prompt shows.
  * `okTitle`*(string)*: the title of confirm button.
  * `cancelTitle`*(string)*: the title of cancel button.
* `callback`*(function)*: callback when complete.
   
This method has a callback function whose arguments will be:

* `e`*(object)*: a event object.
  * `status`*(string)*: the title of the button that clicked by user.
  * `data`*(string)*: the value of the text that entered by user.

#### Example

```javascript
var arg1 = 'I am prompt!'
var arg2 = 'I am ok'
var arg3 = 'I am cancel'
var modal = require('@weex-module/modal');
modal.prompt({
  message: arg1,
  okTitle: arg2,
  cancelTitle: arg3
}, function(e) {
  nativeLog(e.status + ', ' + e.data);
  // TODO after the prompt is complete.
});
```



