## weex-html5

### Intro

Weex is a solution for developers to build a world-class projects on multiple platforms (include both native and web platforms) based on html-css-javascript liked domain specific language. Weex is focused on lightweight, extendable and high performance, and the finally destination -- write once, run anywhere.

The dot we code transformed to weex bundle can currently run on android, ios and web platform. Weex HTML5 is a renderer for weex bundle to run in a webview or a browser environment.

### Get Weex HTML5

Use npm to install the latest version of Weex HTML5, require it from your code in the commonJS way and use it as a npm package. You should require Weex js-framework as well, which is as a dependency to Weex HTML5, or you can import Weex HTML5 from the javascript file as in the node modules' path ``node_modules/weex-html5/dist/``, as in this way you don't need to require Weex js-framework manually anymore since it is already embeded.

#### Install from npm

Make sure you get the latest version by ``npm install`` or ``npm update``. Want more info about npm cmd? Please checkout the [npm official site](https://docs.npmjs.com/).

```
npm install weex-html5
```

#### 1. require weex-html5.

```
require('weex-js-framework')
var weex = require('weex-html5');
```

#### 2. require the total-bundle.

This bundle includes both ``weex-js-framework`` and ``weex-html5``.

```
require('./node_modules/weex-html5/dist/bundle')
```

### Initialize

You can initialize weex through the API ``init``. This method takes a config object as argument to confirm the runtime infomation and environment. Following parameters can be set by this config object:

* ``appId``: app instance id, can be either a string or a number
* ``source``: the requested url of weex bundle, or the transformed code it self.
* ``loader``: loader to load the request weex bundle, whose value is from 'xhr' or 'jsonp' or 'source'.
  * ``xhr``: load the source (weex bundle url) by XHR
  * ``jsonp``: load the source bundle by JSONP
  * ``source``: the source parameter above should be a weex bundle content (transformed bundle).
* ``rootId``: id of the root element. Default value is 'weex'.

Here is a example to do the initialzation:

```
function weexInit() {
  function getUrlParam (key) {
    var reg = new RegExp('[?|&]' + key + '=([^&]+)')
    var match = location.search.match(reg)
    return match && match[1]
  }

  var loader = getUrlParam('loader') || 'xhr'
  var page = getUrlParam('page')

  // jsonp callback name should be specified or be the default
  // value 'weexJsonpCallback' if the 'jsonp' loader is used.
  var JSONP_CALLBACK_NAME = 'weexJsonpCallback'

  window.weex.init({
    jsonpCallback: JSONP_CALLBACK_NAME,
    appId: location.href,
    source: page,
    loader: loader,
    rootId: 'weex'
  })
}

weexInit()
```

### Initialize with multiple instances

```
var weexConfig = [{
  appId: 'weexCt1',
  source: '//your-source-provider/bundle1.js',
  loader: 'xhr',
  width: document.querySelector('#weexCt1').getBoundingClientRect().width,
  rootId: 'weexCt1'
}, {
  appId: 'weexCt2',
  source: '//your-source-provider/bundle2.js',
  loader: 'xhr',
  width: document.querySelector('#weexCt2').getBoundingClientRect().width,
  rootId: 'weexCt2'
}]

function weexInit() { 
  // Init with multiple config objects for multiple instances.
  window.weex.init(weexConfig)
}

weexInit()
```

### use weex-toolkit to preview Demo

install weex-toolkit:

```
npm install -g weex-toolkit
```

run the command ``weex`` to watch the page generated by your ``.we`` file:

```
weex test.we
```

### Develop

#### Directories

**entry files**

* ``index.html``
* ``main.js``

**source code files**

* ``api`` directory: implementation of APIs.
* ``bridge`` directory:
  * ``sender.js``: encapsulate the two types of callJS method (fireEvent and callback).
  * ``receiver.js`` export the methods callNative and nativeLog to global.
* ``components`` directory: implementation of native components (container, text, image, list, slider, tabheader etc.).
* ``weex.js`` export weex to the global. Do the initiation.
* ``protocol.js``
  * registeration of components.
  * registeration of APIs.
* ``loader.js`` implementation of three different type of loaders (mtop/xhr/source).
* ``FrameUpdater.js`` execute tasks throuth requestAnimationFrame.
* ``config.js`` configuration.

#### Build

to build a bundle.js includes both ``weex-js-framework`` and ``weex-html5`` to the path``./dist/bundle.js``:

```
gulp
```

to build a singular weex.js inlcudes only ``weex-html5`` to path ``./dist/weex.js``:

```
gulp weex
```

#### Test

You can use casperjs to lauch e2e test. The test cases are writen in these paths: ``test/e2e/*.js``

```
tnpm install -g casperjs
casperjs test style.js
```


